#!/usr/bin/python
#-*- coding:utf-8 –*-

import re #regular expression
import os
import sys
import glob
import StringIO
import itertools

'''
转换存储过程中的变量定义,对于:=的模式赋初值，改为set,default方式默认值保持不变
'''
def convertVarDefine(dbtype,contents):
    returnVal = ""
    needsetvar = ""
    if dbtype == 'oracle':
        varDefineContents = re.finditer('create.*procedure[\s\S]*?is([\s\S]*?)begin',contents,re.I|re.M)
        for match in varDefineContents:
            returnVal = match.group(1).strip('\n')
            allvars = re.finditer('^[\s]*([^-][^-].)*?;',returnVal,re.I|re.M)
            for var in allvars:
                oldc = var.group()
                newc = 'DECLARE '+var.group().strip().split(':=')[0].strip(';')
                newc = re.sub('varchar2','VARCHAR',newc,0,re.I|re.M)
                newc = re.sub('([0-9]+)[\s]+char','\\1',newc,0,re.I|re.M)
                newc = re.sub('[\s]+',' ',newc,0,re.I|re.M)
                newc = re.sub('[\s]+number',' DECIMAL',newc,0,re.I|re.M)
                newc = newc.strip() + ';' 
                returnVal = returnVal.replace(oldc,newc)
                if ':=' in var.group(): #判断是否有:=格式默认值，如果有则转换到set
                    needsetvar = needsetvar + 'SET '+var.group().strip().split(' ')[0].split('\t')[0] + ' = ' + var.group().strip().split(':=')[1].strip() + '\n'           
        returnVal= returnVal + '\n' + needsetvar.strip()          
    return returnVal

'''
转换存储过程中create语句,替换create or replace,修改in、out的参数位置，添加delimiter，drop procedure语句
'''
def convertCreateProc(dbtype,contents):
    returnVal = ""
    params = ""
    if dbtype == 'oracle':
        varDefineContents = re.finditer('^[\s]*create[\s]+or[\s]+replace[\s]+procedure[\s]+([\S]+)[\s]*\(([\s\S]*?)\)[\s]*.*is',contents,re.I|re.M)
        for match in varDefineContents:
            procName = match.group(1).strip('\n')
            paramLists = match.group(2).strip('\n').split(',')
            for param in range(len(paramLists)):
                paramin = ""
                paramout = ""
                if ' in ' in paramLists[param].strip().lower():
                    paramin = re.sub('[\s]+in[\s]+',' ',paramLists[param].strip(),0,re.I|re.M)
                    paramin = 'IN ' + paramin + ','
                if ' out ' in paramLists[param].strip().lower():
                    paramout = re.sub('[\s]+out[\s]+',' ',paramLists[param].strip(),0,re.I|re.M)
                    paramout = 'OUT ' + paramout + ','
                params =  params + paramin + paramout
                params = re.sub('varchar2','VARCHAR',params,0,re.I|re.M)

    returnVal = "DELIMITER //\nDROP PROCEDURE IF EXISTS " + procName + " //\nCREATE PROCEDURE " + procName + "(" + params.strip(',') + ")\nBEGIN\n"
    return returnVal

'''
转换EXECUTE IMMEDIATE
'''
def convertExecuteImmediate(dbtype,contents):
    if dbtype == 'oracle':
        executeImmContents = re.finditer('EXECUTE[\s]+IMMEDIATE([\s\S]*?;)',contents,re.I|re.M)
        for match in executeImmContents:
            executecontent = match.group(1).strip('\n')
            executecontent = re.sub("[\s]*'[\s]*DROP[\s]+STORAGE[\s]*'","''",executecontent,0,re.I|re.M)

    returnVal = "set @SQL = " + executecontent + "\nprepare stmt from @SQL;\nexecute stmt;\n"
    return returnVal

'''
转换变量赋值，转换范围是单个存储过程的begin与end存储过程段
'''
def convertAssignment(dbtype,contents):
    if dbtype == 'oracle':
        assignmentContents = re.finditer('(^.*):=(.*;)',contents,re.I|re.M)
        for match in assignmentContents:
            oldAssignment = match.group()
            newAssignment = "set " + match.group(1) + " = " + match.group(2)
            contents = contents.replace(oldAssignment,newAssignment)
    return contents
    
'''
函数名：getFunContent
入参：第一个参数是查找的源字符串，第二个参数是在字符串要查找的函数名称，第三个参数是是否带有函数名称，1不带，0带
作用：获取字符串中函数的内容
返回值：string
使用方式：for index in range(len(getFunContent(contents,'to_date'，1))):
              print 'to_date'+getFunContent(contents,'to_date',1)[index];
'''
def getFunContent(sourceStr,funcName,onlyColumn):
    returnContents="";
    funcCotents=re.finditer(funcName,sourceStr,re.I|re.M);
    for match in funcCotents:
        funcstr=''.join(match.group());
        startPos=match.span()[0]+len(funcstr);
        leftParenthesesCnt=1;
        loopCnt=0;
        while True:
            if sourceStr[loopCnt+startPos]=='(':
                leftParenthesesCnt=leftParenthesesCnt+1;
                if leftParenthesesCnt==1:
                    break;
            elif sourceStr[loopCnt+startPos]==')':
                leftParenthesesCnt=leftParenthesesCnt-1;
                if leftParenthesesCnt==1:
                    break;
            loopCnt=loopCnt+1;
        returnContents=returnContents+sourceStr[startPos:startPos+loopCnt+1]+'&';
    if (onlyColumn == 1):
        return ''.join(returnContents.strip('&').split('&'));
    elif (onlyColumn == 0):
        return funcstr+''.join(returnContents.strip('&').split('&'));
    else:
        return "param value not support!"

'''
清除不用的语句，比如commit，rollback
'''
def clearCommonContents(dbtype,contents):
    contents = re.sub("commit[\s]*;[\s]+","",contents,0,re.I|re.M)
    
    return contents

'''
SQL exception 处理
'''
def convertException(dbtype,contents):
    
    return 

'''
函数名：convCommonGrammar
入参：参数是查找的源字符串
作用：转换异构数据库到GBase 8a的部分语法差异
返回值：转换后的字符串
'''
def convCommonGrammar(sourceStr):
    #转换双横线的注释，由原来的不带空格转换为--加空格模式
    sourceStr=re.sub('([^\'][\s]*)--','\\1-- ',sourceStr,0,re.I|re.M);
    #去掉no log
    sourceStr=re.sub('([^\'][\s]*)--','\\1-- ',sourceStr,0,re.I|re.M);
    #去掉no fallback关键字，包括前后的逗号
    sourceStr=re.sub(',?[\s]*no[\s]+fallback[\s]*,?','',sourceStr,0,re.I|re.M); 
    #去掉no before\after journal关键字，包括前后的逗号
    sourceStr=re.sub(',?[\s]*no[\s]+[\S]*[\s]+journal[\s]*,?','',sourceStr,0,re.I|re.M);
    #去掉CHECKSUM = DEFAULT,
    sourceStr=re.sub(',?[\s]*checksum[\s]*=[\s]*[\S]+[\s]*,?','',sourceStr,0,re.I|re.M);
    #去掉DEFAULT MERGEBLOCKRATIO
    sourceStr=re.sub(',?[\s]*default[\s]+MERGEBLOCKRATIO[\s]*,?','',sourceStr,0,re.I|re.M);
    #在create table之间去掉set关键字
    sourceStr=re.sub('(create.*[\s]+)set[\s]+(.*table)','\\1\\2',sourceStr,0,re.I|re.M);
    #在create table之间去掉multiset关键字
    sourceStr=re.sub('(create.*[\s]+)multiset[\s]+(.*table)','\\1\\2',sourceStr,0,re.I|re.M);
    #在create table之间转换volatile到temporary关键字关键字
    sourceStr=re.sub('(create.*[\s]+)volatile[\s]+(.*table)','\\1 TEMPORARY \\2',sourceStr,0,re.I|re.M);
    #替换TITLE关键字为COMMENT，需要把关键字前后的空格一起搜索
    sourceStr=re.sub('[\s]+title[\s]+',' COMMENT ',sourceStr,0,re.I|re.M);
    #替换CHARACTER SET .* CASESPECIFIC为空
    sourceStr=re.sub('[\s]+CHARACTER[\s]+SET.*CASESPECIFIC','',sourceStr,0,re.I|re.M);
    #去掉DATE FORMAT 'YYYYMMDD'中的FORMAT xxx
    sourceStr=re.sub("[\s]+DATE[\s]+FORMAT[\s]+'YYYYMMDD'",' DATE',sourceStr,0,re.I|re.M);
    
    return sourceStr

